// Zoom Search CGI Front End.Demo software
// Main module.
// Copyright Wrensoft 2006-2010
// www.wrensoft.com
//
// Demo software to show how an external software application can use the Zoom search.cgi
// function to search an index of documents. This will be useful for CD/DVD searches and
// searches of local hard drives (and in any case when a web server is not available).
//
// If you are planning to use Zoom for searching a web site on the internet
//
// The Zoom indexer (see, www.wrensoft.com) should be used to create an index of documents
// This 'front end' software (Zoom FE) can then be used to call the search.cgi executable
// supplied with Zoom and collect the results for display.
//
// User data ==> ZoomFE ===> Search.cgi <== Index files
//                              ||
//                              \/
//                           ZoomFE
//                              ||
//                              \/
//                Search Results back to the user
//
//

#define	MAIN_MODULE
 

#include <windows.h>
#include <stdio.h>
#include <process.h>
#include <ERRNO.H>
#include <atltypes.h>
#include <gdiplus.h>
#include <Exdisp.h>

#include <stdlib.h>
#include <crtdbg.h>
#include <shlwapi.h>
#include "resource.h"
#include "CGIFrontEnd.h"
#include "WebControlEvents.h"
#include "WebControl.h"


static HINSTANCE    ghInstance;
static char			*SearchResults;											//Pointer to a buffer to hold the search results
static char			*TmpResultFileName = "CGIFrontEndSearchResult.html";	//Name of temp file used to hold search results
static char			*SearchCGIFileName = "search.cgi";						//Name of search executable to call
static char			*ErrorCaption = "Zoom CGIFrontEnd Error";
#define _CRTDBG_MAP_ALLOC
//Variable to store window coordinates
static  int				cxOld;
static  int				cyOld;

//Gdi
Gdiplus::Graphics *gs;
Gdiplus::Image *image ;

//Local global web control
CWebCtrl*	htmlWindow;

//
// Function protypes
//
void OnSize (HWND hwnd, int x, int y);
BOOL CALLBACK MainDlgProc	(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK AboutDlgProc	(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam);
int OnMaxMinInfo (HWND hwnd, LPMINMAXINFO WinInfo);
void Cleanup(HWND hDlg);
void OnVisitWebsite(HWND hDlg);
void OnViewInBrowser (HWND hDlg);
void PerformSearch (HWND hDlg);
void OnCreate (char *skipOffset); 
void BuildEnvironmentFromDocument (HWND hDlg);
void BuildEnvironmentPage (HWND hDlg);
void BuildEnvironmentFromString(char* urlString);
int  InitDialog (HWND hDlg);
void CreateHTMLControl (HWND hDlg);
void ConvertURLtoPath (char *URLPath);


/********************************/
//WinMain
//
//Entry point
/********************************/
int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG		msg;
	
	ghInstance = hInstance;

	//Get the applications directory
	GetModuleFileName (NULL, RunDirName, sizeof(RunDirName));
	*strrchr (RunDirName, '\\') = '\0';

	// Create and display the window	
	ghMainWnd = CreateDialog (hInstance, MAKEINTRESOURCE(IDD_CGIFRONTEND), 0, MainDlgProc);
	if (InitDialog (ghMainWnd) == NOK)
		return -1;
	ShowWindow (ghMainWnd, nCmdShow) ;
	// Set the window icon
	SetClassLongPtr(ghMainWnd, GCLP_HICON, (LONG_PTR)LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1)));

    while (GetMessage (&msg, NULL, 0, 0) )
    {
        if (!IsDialogMessage (ghMainWnd, &msg) )
        {
            TranslateMessage ( & msg );
            DispatchMessage ( & msg );
        }
    }

	//Memory leak detection
	_CrtDumpMemoryLeaks();
	return msg.wParam ;
}


/********************************/
//OnMaxMinInfo
//
//Stop dialog being too small
/********************************/
int OnMaxMinInfo (HWND hwnd, LPMINMAXINFO WinInfo)
 {

	WinInfo->ptMinTrackSize.x = MIN_WINDOW_WIDTH;
	WinInfo->ptMinTrackSize.y = MIN_WINDOW_HEIGHT;
	return 0;
}

/********************************/
//SwapToBackSlashes
//
// Swaps '/' for '\'
/********************************/
void SwapToBackSlashes(TCHAR* string)
{
	TCHAR* ch;
	ch = string;
	while (*ch) {
		if (*ch == '/')
			*ch = '\\';
		ch++;
	}
}

/********************************/
//UpdateControl
//
//	Move or resize a control (without any relative reference to any other controls on the window)
//		PT_MOVE:		Move the control down and across, keeping the  same size
//		PT_MOVE_DOWN:	Move the control down only (not across), keeping the  same size
//		PT_MOVE_ACROSS:	Move the control across only (not down), keeping the  same size
//		PT_RESIZE:		Move the controls bottom right co-ordinate, keeping the top left the same
//		PT_RESIZE_HALFY:Move the controls bottom right co-ordinate (x and 0.5y), keeping the top left the same
/********************************/
void UpdateControl(HWND hDlg, int iControl, int cxDelta, int cyDelta, int iType)
{
	HWND	hImageFrame;
	RECT	rcImageFrame;
	POINT	ptImageFrameTL;
	POINT	ptImageFrameBR;
	int		cxFrameWidth	= 0;
	int		cyFrameHeight	= 0;

	hImageFrame = GetDlgItem(hDlg, iControl);

	GetWindowRect(hImageFrame, &rcImageFrame);

	// Get current top/left coordinates
	ptImageFrameTL.x = rcImageFrame.left;
	ptImageFrameTL.y = rcImageFrame.top;
	ScreenToClient(hDlg, &ptImageFrameTL);
	// Get current bottom/right coordinates
	ptImageFrameBR.x = rcImageFrame.right;
	ptImageFrameBR.y = rcImageFrame.bottom;
	ScreenToClient(hDlg, &ptImageFrameBR);
	// Work out width/height
	cxFrameWidth  = ptImageFrameBR.x - ptImageFrameTL.x;
	cyFrameHeight = ptImageFrameBR.y - ptImageFrameTL.y;	

	if (iType == RS_MOVE)							//Move the control down and across, keeping the  same size
		MoveWindow( hImageFrame, 
					ptImageFrameTL.x + cxDelta, 
					ptImageFrameTL.y + cyDelta, 
					cxFrameWidth,  
					cyFrameHeight, 
					FALSE);
	else if (iType == RS_RESIZE)					//Move the controls bottom right co-ordinate, keeping the top left the same
		MoveWindow( hImageFrame, 
					ptImageFrameTL.x, 
					ptImageFrameTL.y, 
					cxFrameWidth  + cxDelta,  
					cyFrameHeight + cyDelta, 
					FALSE);
	/*
	else if (iType == RS_RESIZE_HALFY)					//Move the controls bottom right co-ordinate, keeping the top left the same
		MoveWindow( hImageFrame, 
					ptImageFrameTL.x, 
					(long) ((float)ptImageFrameTL.y/2.0), 
					cxFrameWidth  + cxDelta,  
					cyFrameHeight + cyDelta, 
					FALSE);
					*/
	else if (iType == RS_MOVE_DOWN)					//Move the control down only (not across), keeping the  same size
		MoveWindow( hImageFrame, 
					ptImageFrameTL.x, 
					ptImageFrameTL.y + cyDelta, 
					cxFrameWidth,  
					cyFrameHeight, 
					FALSE);
	else if (iType == RS_MOVE_ACROSS)				//Move the control across only (not down), keeping the  same size
		MoveWindow( hImageFrame, 
					ptImageFrameTL.x + cxDelta, 
					ptImageFrameTL.y, 
					cxFrameWidth,  
					cyFrameHeight, 
					FALSE);
	else if (iType == RS_MOVE_ACROSS_SIZE_DOWN) //Move across and resize down
		MoveWindow( hImageFrame, 
					ptImageFrameTL.x + cxDelta, 
					ptImageFrameTL.y, 
					cxFrameWidth,  
					cyFrameHeight + cyDelta, 
					FALSE);
	else if (iType == RS_MOVE_DOWN_SIZE_ACROSS) //Move across and resize down
		MoveWindow( hImageFrame, 
					ptImageFrameTL.x, 
					ptImageFrameTL.y + cyDelta, 
					cxFrameWidth + cxDelta,  
					cyFrameHeight, 
					FALSE);
	else if (iType == RS_MOVE_DOWN_AND_ACROSS) //Move across and resize down
		MoveWindow( hImageFrame, 
					ptImageFrameTL.x + cxDelta, 
					ptImageFrameTL.y + cyDelta, 
					cxFrameWidth,  
					cyFrameHeight, 
					FALSE);
	else if (iType == RS_SIZE_ACROSS) //Move across and resize down
		MoveWindow( hImageFrame, 
					ptImageFrameTL.x, 
					ptImageFrameTL.y, 
					cxFrameWidth + cxDelta,  
					cyFrameHeight, 
					FALSE);

}
/********************************/
//OnSize
//
//	Function to move controls on window resize
/********************************/
void OnSize(HWND hwnd, int x, int y)
{

	int	cxDelta	= 0;
	int	cyDelta	= 0;

	// Get the x and y deltas
	cxDelta = x - cxOld;
	cyDelta = y - cyOld;

	// Resize or move the controls
	UpdateControl(hwnd, IDC_ABOUT, cxDelta,  cyDelta, RS_MOVE);
	UpdateControl(hwnd, IDOK, cxDelta,  cyDelta, RS_MOVE);
	UpdateControl(hwnd, IDC_WEBSITE, cxDelta,  cyDelta, RS_MOVE);
	UpdateControl(hwnd, IDC_SEARCH, cxDelta,  cyDelta, RS_MOVE);

	//backup coordinates for next move
	cxOld = x;
	cyOld = y;

	//Redraw
	InvalidateRect(hwnd, NULL, FALSE);
}

/********************************/
// WndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
//
// Windows callback procedure.
// Handles all the messages for the dialog box
/********************************/

BOOL CALLBACK MainDlgProc (HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) 
{	
	int width, height;
	Gdiplus::Unit  units;
	CRect rect;
	WCHAR filename[200] = (L"logo.png");
	ULONG_PTR	m_gdiplusToken;

	switch (iMsg)
	{
		
		//This message is sent when a URL is clicked that links to the zoom CGI executable
		//or another outside link
	case WM_APP:
		{
			//Check if we need to execute the cgi again
			if(	strncmp(lastURL, "about:blanksearch.cgi?", 22) == 0 ||
				strncmp(lastURL, "about:search.cgi?", 17) == 0)
			{
				BuildEnvironmentFromString(lastURL);
				PerformSearch(hDlg);
			}
			else //otherwise load the url normally
			{
				//Trim off the #search= part of the URL which can be present for PDF documents
				//when PDF highlighting is on. This seems to only work via HTTP and fails completely in
				//old versions of the Acrobat reader.
				char *PDFString;
				PDFString = strstr (lastURL, "#search=");
				if (PDFString != NULL)
					*PDFString = '\0';
								
				// For Firefox, we need to find an absolute path to the file (relative paths won't work)
				if (_tcsnicmp(lastURL, _T("http://"), 7) != 0 &&
					_tcsnicmp(lastURL, _T("https://"), 8) != 0 &&
					_tcsnicmp(lastURL, _T("file://"), 7) != 0 &&
					PathIsRelative(lastURL))
				{
					char tmpURL[MAX_URL_LEN+1];
					strcpy_s(tmpURL, MAX_URL_LEN, lastURL);
					PathCombine(lastURL, RunDirName, tmpURL);
					SwapToBackSlashes(lastURL);				
				}

				//Open the page or document that the user has clicked on
				int ret = 0;
				ret = (int)ShellExecute (ghMainWnd, "open", lastURL, NULL, RunDirName, SW_SHOWDEFAULT  );
				
				if(ret == ERROR_FILE_NOT_FOUND  || ret == ERROR_PATH_NOT_FOUND)
				{
					char tmpMsg[MAX_URL_LEN + MAX_PATH_LEN];
					sprintf_s(tmpMsg, MAX_URL_LEN + MAX_PATH_LEN, "Unable to find file\n%s\nin directory\n%s", lastURL, RunDirName);
					MessageBox (ghMainWnd, tmpMsg,"File not found", MB_OK | MB_ICONEXCLAMATION);
				}
				else if (ret <= 32) 
				{
					char tmpMsg[MAX_URL_LEN + MAX_PATH_LEN];
					sprintf_s (tmpMsg, MAX_URL_LEN + MAX_PATH_LEN, "Could not start your default internet browser.\n"\
						"Start your browser manually and use the address..\n"\
						"%s\n", lastURL);
					
					MessageBox (ghMainWnd, tmpMsg,"Browser Error", MB_OK | MB_ICONEXCLAMATION);
				}
				
			}
			return(0);
		}
	//This message is sent when a search button is clicked
	case WM_APP+1:	
			BuildEnvironmentFromDocument (hDlg);
			PerformSearch (hDlg);
			break;
		
	case WM_GETMINMAXINFO:
			OnMaxMinInfo (hDlg, (LPMINMAXINFO) lParam);
			break;

	case WM_SIZE:
			OnSize (hDlg, LOWORD (lParam), HIWORD (lParam));
			width = LOWORD(lParam) - WEBCTRL_WIDTHOFFSET_X;
			height = HIWORD(lParam) - WEBCTRL_HEIGHTOFFSET_X;
			htmlWindow->Resize(WEBCTRL_OFFSET_X, WEBCTRL_OFFSET_Y, width, height);
			break;
	
	case WM_SHOWWINDOW :
			BuildEnvironmentPage (hDlg);
			PerformSearch (hDlg);
			break;

	case WM_PAINT :
		{
			//Draw the logo on top right corner
			RECT ClientRect;
			float aspect_r = 1.0;
			GetClientRect (hDlg, &ClientRect) ;
			//Start GDI
			Gdiplus::GdiplusStartupInput gdiplusStartupInput;
			Gdiplus::GdiplusStartup(&m_gdiplusToken, &gdiplusStartupInput, NULL);

			//Instantiate graphics and image object
			gs = new Gdiplus::Graphics(hDlg);	
			image = new	Gdiplus::Image(filename);

			if (image != NULL)
			{
				Gdiplus::RectF destRect, srcRect;
				image->GetBounds(&srcRect, &units);

				//Calculate aspect ratio if image fail to fit in
				if (srcRect.Height > IMAGE_HEIGHT) {
					aspect_r = float(IMAGE_HEIGHT/srcRect.Height);
				}
				//Set destination rectangle for image on screen
				//The aspect ratio shall be locked while resizing image unless height is less then IMAGE_HEIGHT
				if (ClientRect.right > (srcRect.Width*aspect_r + IMAGE_OFFSET))
					destRect.X = ClientRect.right - (srcRect.Width*aspect_r + IMAGE_OFFSET);
				else
					destRect.X = IMAGE_OFFSET;

				destRect.Y = IMAGE_OFFSET;

				if (ClientRect.right > (srcRect.Width*aspect_r + IMAGE_OFFSET))
					destRect.Width = srcRect.Width*aspect_r;
				else
					destRect.Width = (float)(ClientRect.right - 2*IMAGE_OFFSET);

				if(srcRect.Height > IMAGE_HEIGHT)
					destRect.Height = IMAGE_HEIGHT;
				else
					destRect.Height = srcRect.Height;

				//Draw the image
				gs->DrawImage(image, destRect, srcRect.X, srcRect.Y, srcRect.Width,	srcRect.Height,	Gdiplus::UnitPixel, NULL);

			}
			//shutdown Gdi
			Gdiplus::GdiplusShutdown(m_gdiplusToken);
		}

	case WM_COMMAND :
		switch (LOWORD (wParam)) 
		{
		case IDC_ABOUT:
			DialogBox (ghInstance, MAKEINTRESOURCE(IDD_ABOUT), hDlg, AboutDlgProc);
			break;

		case IDC_WEBSITE:
			OnVisitWebsite(hDlg);
			break;
		case IDC_SEARCH:
			PostMessage(hDlg, WM_APP+1, 0, 0);
			break;
		
		case IDCANCEL:
		case IDOK:
			Cleanup(hDlg);
			DestroyWindow (hDlg);
			return TRUE;
		}
		return 0 ;
		
	case WM_DESTROY :
		PostQuitMessage (0) ;
		return 0 ;
	}
	
	return FALSE;
}


/********************************/
//InitDialog
//
//Called just after the dialog is created
/********************************/
int InitDialog (HWND hDlg)
{
	int xoffset, yoffset, dlgWidth, dlgHeight;
	RECT dlgRect;

	//Allocate a buffer for the search results.
	SearchResults = (char *) malloc (MAX_RESULT_BUFF_LEN);
	if (!SearchResults) {
		MessageBox (NULL, "Could not allocate enough RAM", ErrorCaption, MB_ICONEXCLAMATION | MB_OK);
		return (NOK);
	}
	memset (SearchResults, 0, MAX_RESULT_BUFF_LEN);
	
	//center dialog
	//Get the current dialog size
	GetWindowRect(hDlg, &dlgRect);	
	dlgWidth = dlgRect.right-dlgRect.left;
	dlgHeight = dlgRect.bottom-dlgRect.top;

	//Store current position of screen
	RECT		rect;
	GetClientRect(hDlg, &rect);
	cxOld = rect.right;
	cyOld = rect.bottom;

	//Get x & y start points based on screen size
	yoffset = GetSystemMetrics (SM_CYFULLSCREEN) / 2 - (dlgHeight / 2);
	xoffset = GetSystemMetrics (SM_CXFULLSCREEN) / 2 - (dlgWidth / 2);
	
	SetWindowPos(hDlg, HWND_TOP,xoffset, yoffset, dlgWidth, dlgHeight, 0); 
	
	//Create the broswer 
	CreateHTMLControl (hDlg);
		
	return (OK);
}

//********************************
//Cleanup
//
//
//********************************
void Cleanup(HWND hDlg)
{
	free (SearchResults);

	if(htmlWindow != NULL)
		delete(htmlWindow);
}



//********************************
//OnVisitWebsite
//
//Visit the passmark web site
//********************************
void OnVisitWebsite(HWND hDlg) {
	
	if ((int) ShellExecute (hDlg, "open", "http://www.wrensoft.com",
		NULL, ".", SW_SHOWMAXIMIZED ) <= 32) {
		MessageBox (hDlg,
			"Could not start your default internet browser.\n"\
			"Start your browser manually and use the address..\n"\
			"http://www.wrensoft.com/", "No default browser", MB_ICONEXCLAMATION | MB_OK);
	}
}


void OnCreate (char *skipOffset) 
{
	//Load HTML results into IE window
	htmlWindow->WriteToHtmlDoc(skipOffset);

}

//********************************
//PerformSearch
//
//Builds up the environment and does a serch
//by calling the search.cgi
//********************************
void PerformSearch (HWND hDlg) 
{
	char* skipOffset;
	HANDLE	StdOutFromSearch;
	char	TmpPath[MAX_PATH_LEN];
	DWORD	nBytesRead;
	int		CurrBufOffset = 0;
	int		LastErr;

	//Clear buffer
	memset (SearchResults, 0, MAX_RESULT_BUFF_LEN);

	//Build file name for temp file in current directory
	sprintf_s (TmpPath, MAX_PATH_LEN, "%s\\%s", RunDirName, SearchCGIFileName);

	//Launch the search.cgi executable with it output (stdout)
	// redirected into a pipe that can be read by this process
	StdOutFromSearch = SpawnWithRedirect (TmpPath);

	//Read the input from the pipe.
	while(StdOutFromSearch)
	{
		if (!ReadFile(StdOutFromSearch,&SearchResults[CurrBufOffset],MAX_RESULT_BUFF_LEN - CurrBufOffset, &nBytesRead,NULL))
		{
			LastErr = GetLastError();
            if (LastErr == ERROR_BROKEN_PIPE || LastErr == ERROR_NOACCESS)
				break; // pipe done - normal exit path.
			else {
				MessageBox (NULL, "Read error", "Error", MB_OK);
				break;
			}
		}

		//Check if that was the end of the data
		if (!nBytesRead)
			break;
		CurrBufOffset += nBytesRead;
		if (CurrBufOffset >= MAX_RESULT_BUFF_LEN)
			break;
	}
	
	//find the first newline char and that portion of the string
	//to skip the content type / charset info
	skipOffset = strchr(SearchResults, '\n');
	
	//make sure isn't null
	if(skipOffset == NULL)
		skipOffset = SearchResults;

	//Convert URLs back to ASCII, i.e. remove %20 and replace by space
	ConvertURLtoPath (skipOffset);

	//Load HTML results into IE window
	htmlWindow->WriteToHtmlDoc(skipOffset);

	 //Write some html to the window, clearing the window first

	if (StdOutFromSearch)
		CloseHandle (StdOutFromSearch);
}

//********************************
//BuildEnvironmentFromString
//
//Build the environment expected by the search.cgi executable
//by getting the values from the passed string and setting the 
//QUERY_STRING value
//********************************
void BuildEnvironmentFromString(char* urlString)
{
	char	EnvString[MAX_ENVIRONMENT_STRING_LEN];
	//find first ? and save split string
	char* envStart = strchr(urlString, '?');
	if(envStart != NULL)
	{
		//Build the QUERY_STRING that will be passed to search.cgi
		sprintf_s (EnvString, MAX_ENVIRONMENT_STRING_LEN, "QUERY_STRING=%s",envStart+1);
	}
	
	//The environment from this process will be passed to the search.cgi process,
	// so load up this process with the required name / value pairs.
	_putenv ("REQUEST_METHOD=GET");
	_putenv (EnvString);
	
}
//********************************
//BuildEnvironmentPage
//put Get request to search script when windows show for first time
//********************************
void BuildEnvironmentPage (HWND hDlg) {

	
	_putenv ("REQUEST_METHOD=GET");

}

//********************************
//BuildEnvironmentFromDocument
//
//Build the environment expected by the search.cgi executable
//by getting the values entered by the user int the html doc and 
//setting the QUERY_STRING value
//********************************
void BuildEnvironmentFromDocument (HWND hDlg) {

	char	SearchString[MAX_SEARCH_STRING_LEN]="";
	char	MetaString[MAX_SEARCH_STRING_LEN]="";
	char	HiddenStr[MAX_SEARCH_STRING_LEN]="";
	char	SearchStringEnc[MAX_SEARCH_STRING_LEN];
	char	*CurrValSrc, *CurrValDest;
	char	EnvString[MAX_ENVIRONMENT_STRING_LEN];
	char	NumResults[20];

	int		AllWord;
	int		nDestCount;


	//Function to parse the user input in MSHTML 
	htmlWindow->ParseHTML (HiddenStr, SearchString, NumResults, &AllWord, MetaString);
	CurrValSrc = SearchString;
	CurrValDest = SearchStringEnc;
	nDestCount = 0;
	while (*CurrValSrc) 
	{
		switch (*CurrValSrc) {
		case ' ':
			*CurrValDest = '+';
			break;
		case '%':
		case '"':
		case '&':
		case '+':
			sprintf_s (CurrValDest, MAX_SEARCH_STRING_LEN-nDestCount, "%%%x", *CurrValSrc);
			CurrValDest += 2;
			nDestCount += 2;
			break;
		default:
			*CurrValDest = *CurrValSrc;
		}
		CurrValDest++;
		CurrValSrc++;
		nDestCount++;
	}
	*CurrValDest = '\0';

	//Build the QUERY_STRING that will be passed to search.cgi
	sprintf_s (EnvString, MAX_SEARCH_STRING_LEN, "QUERY_STRING=%s&zoom_query=%s&zoom_per_page=%s&zoom_and=%d%s",
		HiddenStr,
		SearchStringEnc,
		NumResults,
		AllWord,MetaString);
	
	//http://holly/cgi-bin/v7/daterangetest/search.cgi?zoom_sort=0&zoom_xml=0&zoom_query=**&zoom_per_page=10&zoom_and=0&zoom_datefrom=01%2F03%2F2014&zoom_dateto=01%2F05%2F2014

	//The environment from this process will be passed to the search.cgi process,
	// so load up this process with the required name / value pairs.
	_putenv ("REQUEST_METHOD=GET");
	_putenv (EnvString);
}


//********************************
//ConvertURLtoPath
//
//Convert a local URL back into a file path that Windows API
//can use to open the file.
//Converts encoded hex values %xx back into normal ASCII
//Paramters:
//	URLPath:	IN / OUT - Takes URL, outputs Path. Path is always less than or equal to
//				in length to the URL
//********************************
void ConvertURLtoPath (char *URLPath) {

	char	*CurrValSrc, *CurrValDest;
	int		Value;

	CurrValSrc = CurrValDest = URLPath;

	while (*CurrValSrc) {
		if (*CurrValSrc == '%' && isdigit(*CurrValSrc+1)) {					//Convert all values that start with a %xx
			Value = -1;
			sscanf_s (CurrValSrc, "%%%2x", &Value);	//Scan hex value
			// exclude non-ascii values. Also, do not decode the double-quot character (%22, aka 34 in dec)
			// because it is needed to maintain correct HTML for the output links
			if (Value > 0 && Value < 255 && Value != 34) 
			{
				*CurrValDest = (char) Value;
				CurrValSrc += 3;					//Skip point over hex value
				CurrValDest++;
				continue;
			}
		}
		*CurrValDest = *CurrValSrc;
		CurrValDest++;
		CurrValSrc++;
	}
	*CurrValDest = NULL;
}


//********************************
//AboutDlgProc
//
//Dialog box handler for the about window
//********************************
BOOL CALLBACK AboutDlgProc (HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) {

	char  tstr[500];

	switch (iMsg)
	{
    case WM_INITDIALOG :
		sprintf_s (tstr, 500, "%s %s Build: %s", PROGRAM_NAME, PROGRAM_VERSION, PROGRAM_BUILD);
		SetDlgItemText (hDlg, IDC_VERSION, tstr);
		return TRUE ;

	case WM_COMMAND :
		switch (LOWORD (wParam))
		{
		case IDCANCEL:
		case IDOK:
			EndDialog (hDlg, 0) ;
			return TRUE ;	
		}
		break;		
	}
     return FALSE;
}


//********************************
//CreateHTMLControl
//
//Create a HTML control in the dialog window
//********************************
void CreateHTMLControl (HWND hDlg) 
{
	RECT dlgRect;
	GetClientRect(hDlg, &dlgRect);

	htmlWindow = new CWebCtrl(hDlg, ghInstance, SW_SHOW, WEBCTRL_OFFSET_X, WEBCTRL_OFFSET_Y, dlgRect.right-dlgRect.left-WEBCTRL_WIDTHOFFSET_X, dlgRect.bottom-dlgRect.top-WEBCTRL_HEIGHTOFFSET_X); 
}

